///////////////////////////////////////////////////////////////////////////////
// Title      : DIOT WIC testbench DUT harness
// Project    : Distributed I/O Tier
///////////////////////////////////////////////////////////////////////////////
// File       : dut_harness.vhd
// Author     : Denia Bouhired <denia.bouhired@cern.ch>
// Company    : CERN (BE-CO-HT)
// Standard   : VHDL
///////////////////////////////////////////////////////////////////////////////
// Description:
// DIOT/WIC testbench - Harness containing an interface talking worldFIP to a
// nanoFIP/DIOT system board DUT running a WIC application
//
///////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2018 CERN
//
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 0.51 (the “License”) (which enables you, at your option,
// to treat this file as licensed under the Apache License 2.0); you may not
// use this file except in compliance with the License. You may obtain a copy
// of the License at http://solderpad.org/licenses/SHL-0.51.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// “AS IS” BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//
///////////////////////////////////////////////////////////////////////////////


`timescale 1ns/1ps

`include "fip_phy_model.svh"
`include "fip_frame.svh"
`include "fip_hardware_model.svh"



// Backplane/System board diagnostics interface
interface backplane_if (); //backplane interface

  wire rstpo_n, diag_sda, diag_scl;
        // TODO ?
        // clocking cb @(posedge clk);

        // endclocking : cb

        //modport tb_wb_from_nf
        //modport tb_wb_to_nf
endinterface


// FMC/System board WB interface
interface wb_if (input bit clk); //wishbone hook
  wire [15:0] wb_dat_in;
  wire [15:0] wb_dat_out;
  wire [9:0] wb_adr;
  wire wb_stb, wclk, wb_we, wb_cyc, wb_ack, wb_rst;

        // TODO ?
        // clocking cb @(posedge clk);

        // endclocking : cb

        //modport tb_wb_from_nf
        //modport tb_wb_to_nf
endinterface

// FMC/System board FIP interface
interface nf_fmc_if (input bit clk); //nanoFIP hook
  wire nf_var1_acc, nf_var2_acc, nf_var3_acc;
  wire nf_var1_rdy, nf_var2_rdy, nf_var3_rdy;
  wire nf_nostat, nf_rst;

  reg [7:0] nf_address = 8'h01;  // Substation data
  wire [2:0] nf_p3lgth;



endinterface

interface tb_fd_hook (input bit clk); //nanoFIP hook
  bit [7:0] payload[0:123]   ;//= '{124{'h00}};
  // reg send_data = 0;
  // wire [7:0] output_data = 8'h00;


  //Fieldrive and WFIP Master model
  FIPBus fip_bus();
  FieldriveFPGA fieldrive();
  FieldriveModel fd_model(.fpga(fieldrive), .fip(fip_bus.fd));
  FipPHY fip_master(.fip_p(fip_bus.bus.p), .fip_n(fip_bus.bus.n));
 //------Define clocking block
  // clocking cb @(posedge clk_40m);
    // input  nf_var3_rdy, nf_var2_rdy nf_var1_rdy
    // output  nf_rst, nf_nostat, nf_var3_acc, nf_var2_acc nf_var1_acc nf_p3lgth
  // endclocking

task automatic sendFIPframe (uint8_t payload[$]); 
    FipFrame fip_frame;
  
    fieldrive.speed = 'h01;  //1Mbps
    fip_master.setSpeed(1000000);
    // fip_master.setVerbose(1);
    
    #10us;      // wait for reset
    fip_frame = FipFrame::makeMF_ID_DAT('h05, DUT_nf_if.nf_address);
    fip_master.sendFrame(fip_frame);
    #50us;      // silence time
    fip_frame = FipFrame::makeMF_RP_DAT(payload);

    //fip_frame = FipFrame::makeMF_RP_DAT('{'h01, 'h02, 'h03, 'h04, 'h05, 'h06, 'h07, 'h08});
    fip_master.sendFrame(fip_frame);
    #100us;      // silence time
    // fip_frame = FipFrame::makeMF_RP_DAT({DUT_nf_if.input_data});
    // fip_master.sendFrame(fip_frame);
    // DUT_nf_if.send_data = 0;
endtask 

function automatic getRxStatus ();
    return fip_master.rx_idle;
endfunction 

task automatic getFIPframe ();
    FipFrame fip_frame;
    fip_frame = FipFrame::makeMF_ID_DAT('h06, DUT_nf_if.nf_address);
    fip_master.sendFrame(fip_frame);
    #500us;
endtask;

task automatic dispFIPframe ();
   // if (!getRxStatus())
        $display("oN INTERFACE Received data is %p", fip_master.frame_data);
endtask 

task automatic storeFIPframe ();
    //if (!getRxStatus())
        payload=fip_master.frame_data;
endtask 
//TODO
// clocking cb @(posedge clk);


// endclocking : cb

endinterface

module dut_harness;


  reg clk_125m_p = 0; // could be bit?
  reg clk_125m_n = 1; // could be bit?
  always #(4ns) clk_125m_p <= ~clk_125m_p;
  always #(4ns) clk_125m_n <= ~clk_125m_n;
  reg clk_40m = 0;
  always #(12500ps) clk_40m <= ~clk_40m;

  wb_if DUT_wb_if(.clk(clk_125m_p));
  nf_fmc_if DUT_nf_if(.clk(clk_40m));
  tb_fd_hook DUT_fd_if(.clk(clk_40m));
  backplane_if pentair_if();

  // power-on-reset
  reg rstpo_n = 0;
  initial begin
    repeat(3) @(posedge DUT_wb_if.wclk);
    rstpo_n = 1;
  end


  //------------------------------------------------
  spec_top DUT (
    .clk_125m_pllref_p_i (clk_125m_p),
    .clk_125m_pllref_n_i (clk_125m_n),
    .dat_i (DUT_wb_if.wb_dat_in[7:0]),
    .dat_o (DUT_wb_if.wb_dat_out[7:0]),
    .adr_o (DUT_wb_if.wb_adr),
    .stb_o (DUT_wb_if.wb_stb),
    .wclk_o(DUT_wb_if.wclk),
    .we_o  (DUT_wb_if.wb_we),
    .cyc_o (DUT_wb_if.wb_cyc),
    .ack_i (DUT_wb_if.wb_ack),
    .rst_o (DUT_wb_if.wb_rst),

    .rstin_o    (DUT_nf_if.nf_rst),
    .nostat_o   (DUT_nf_if.nf_nostat),
    .var3_acc_o (DUT_nf_if.nf_var3_acc),
    .var3_rdy_i (DUT_nf_if.nf_var3_rdy),
    .var2_acc_o (DUT_nf_if.nf_var2_acc),
    .var2_rdy_i (DUT_nf_if.nf_var2_rdy),
    .var1_acc_o (DUT_nf_if.nf_var1_acc),
    .var1_rdy_i (DUT_nf_if.nf_var1_rdy),
    .p3_lgth_o  (DUT_nf_if.nf_p3lgth),
  
    .diag_scl_b (pentair_if.diag_scl),
    .diag_sda_b (pentair_if.diag_sda));
  //------------------------------------------------
  nanofip NF(
    .c_id_i    (4'hc),  // Constructor field
    .m_id_i    (4'ha),  // Model field
    .p3_lgth_i (DUT_nf_if.nf_p3lgth),
    .rate_i    (DUT_fd_if.fieldrive.speed),
    .subs_i    (DUT_nf_if.nf_address), // Station address
    .nostat_i  (DUT_nf_if.nf_nostat),

    .rstin_i  (DUT_nf_if.nf_rst),
    .rston_o  (),  // Reset output
    .rstpon_i (pentair_if.rstpo_n),// Power-on-reset
    .slone_i  (1'b0),   // Disable standalone mode
    .uclk_i   (clk_40m),   // 40MHz clock

    // Signals accessed by variables 1-3
    .var1_acc_i(DUT_nf_if.nf_var1_acc),
    .var2_acc_i(DUT_nf_if.nf_var2_acc),
    .var3_acc_i(DUT_nf_if.nf_var3_acc),
    .var1_rdy_o(DUT_nf_if.nf_var1_rdy),
    .var2_rdy_o(DUT_nf_if.nf_var2_rdy),
    .var3_rdy_o(DUT_nf_if.nf_var3_rdy),

    // Wishbone
    .dat_o (DUT_wb_if.wb_dat_in),
    .dat_i (DUT_wb_if.wb_dat_out),
    .adr_i (DUT_wb_if.wb_adr),
    .stb_i (DUT_wb_if.wb_stb),
    .wclk_i(DUT_wb_if.wclk),
    .we_i  (DUT_wb_if.wb_we),
    .cyc_i (DUT_wb_if.wb_cyc),
    .ack_o (DUT_wb_if.wb_ack),
    .rst_i (DUT_wb_if.wb_rst),

    // Receiver
    .fd_rxcdn_i(DUT_fd_if.fieldrive.fd_rxcdn),
    .fd_rxd_i  (DUT_fd_if.fieldrive.fd_rxd),
    .fd_txer_i (DUT_fd_if.fieldrive.fd_txer),
    .fd_wdgn_i (DUT_fd_if.fieldrive.fd_wdgn),

    // Transmitter
    .fd_rstn_o (DUT_fd_if.fieldrive.fd_rstn),
    .fd_txck_o (DUT_fd_if.fieldrive.fd_txck),
    .fd_txd_o  (DUT_fd_if.fieldrive.fd_txd),
    .fd_txena_o(DUT_fd_if.fieldrive.fd_txena),

    .r_fcser_o (),
    .r_tler_o  (),
    .u_cacer_o (),
    .u_pacer_o (),

    // JTAG interface
    .jc_tms_o(),
    .jc_tdi_o(),
    .jc_tck_o(),
    .jc_tdo_i(1'b0)
);

  pentair_fcm2_model sim_FCM2(
    .clk_i(DUT_wb_if.wclk),
    .rst_n_i(pentair_if.rstpo_n),
    .sda_b(pentair_if.diag_sda),
    .scl_b(pentair_if.diag_scl));                      


endmodule
