#!/usr/bin/python3

###############################################################################
# This work is part of the Distributed I/O Tier project
# Copyright (c) 2018 CERN (www.cern.ch)
# Author: Greg Daniluk <grzegorz.daniluk@cern.ch>
# Released according to the GNU GPL, version 2 or any later version.
###############################################################################

import sys
import copy
from PyQt5.QtWidgets import *
from PyQt5 import QtCore
from ui_dialog import Ui_Dialog
import threading
import time

class TestDialog(QDialog, Ui_Dialog):
    def __init__(self):
        super(TestDialog, self).__init__()
        # setup GUI from Qt Designer
        self.setupUi(self)

        self.show()

##########################################################
##              DiotSlot class definition               ##
##########################################################
class DiotSlot():
    def __init__(self, Dialog, num, idx):
        self.btn_state = []
        self.io_state  = []
        self.btn_test  = []
        #######################
        self.groupBox = QGroupBox(Dialog)
        self.groupBox.setGeometry(QtCore.QRect(20+idx*170, 20, 150, 40+num*30))
        self.groupBox.setTitle("Slot "+str(idx+1))
        for i in range(0, num):
            self.io_state.append(True)
            self.btn_state.append(QPushButton(self.groupBox))
            self.btn_state[i].setGeometry(QtCore.QRect(20, 30+i*30, 30, 30))
            self.btn_state[i].setText(str((i+1)+idx*num))
            self.btn_state[i].setStyleSheet("background-color: green")
            self.btn_test.append(QPushButton(self.groupBox))
            self.btn_test[i].setGeometry(QtCore.QRect(50, 30+i*30, 80, 30))
            self.btn_test[i].setText("Test")
            self.btn_test[i].clicked.connect(lambda state, slot=idx, io=i: self.action_click(slot, io))

    def action_click(self, slot, io):
        print("Button test clicked - slot(" + str(slot) +") io(" + str(io) + ")")
        ## TODO: add here sending new output state over fieldbus
        self.update_state(io, not self.io_state[io])

    def update_state(self, io, new_state):
        self.io_state[io] = new_state
        # and refresh color
        if self.io_state[io] == True:
            self.btn_state[io].setStyleSheet("background-color: green")
        else:
            self.btn_state[io].setStyleSheet("background-color: red")

##########################################################
##               Thread  class definition               ##
##########################################################
class ReadThread (threading.Thread):
    def __init__(self, threadID, name, slots):
        threading.Thread.__init__(self)
        self.threadID = threadID
        self.name     = name
        self.slots    = slots
        self.end_thread = 0

    def run(self):
        ## TODO: put here cyclic reading of all IO state
        while self.end_thread == 0:
            time.sleep(1)
            self.slots[3].update_state(5, not self.slots[3].io_state[5])
            print("thread updated io")

    def stop(self):
        self.end_thread = 1

if __name__ == '__main__':
    app = QApplication(sys.argv)

    #w = TestDialog()
    w = QDialog()
    slots = []
    for i in range (0, 8):
        slots.append(DiotSlot(w, 16, i))
    #ui = Ui_Dialog()
    #ui.setupUi(w)
    w.show()
    thread1 = ReadThread(1, "ReadThread-1", slots)
    thread1.start()
    sys.exit(app.exec_())
    print("Exiting")
    thread1.stop()
    print("thread stopped?")
